{
'*****************************************************************************
' Build 0135
'
' Project:  GDAL Lazarus Bindings
' Purpose:  GDAL Lazarus internal support functions.
' Author:   Frank Warmerdam, warmerdam@pobox.com
' Lazarus Wrapper : Prajuab Riabroy
'*****************************************************************************
' Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
'
' Permission is hereby granted, free of charge, to any person obtaining a
' copy of this software and associated documentation files (the "Software"),
' to deal in the Software without restriction, including without limitation
' the rights to use, copy, modify, merge, publish, distribute, sublicense,
' and/or sell copies of the Software, and to permit persons to whom the
' Software is furnished to do so, subject to the following conditions:
'
' The above copyright notice and this permission notice shall be included
' in all copies or substantial portions of the Software.
'
' THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
' OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
' FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
' THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
' LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
' FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
' DEALINGS IN THE SOFTWARE.
'*****************************************************************************
}
unit gdalcore;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, gdal;


//' ****************************************************************************
//'               Declarations for C API functions.
//' ****************************************************************************

type
  TGDALMajorObject = class
  private
    FObj : TGDALMajorObjectH;
    //FOwned : boolean;
  public
    //property  Owned : boolean read FOwned write FOwned default false;
    property  Handle : TGDALMajorObjectH read FObj write FObj;
    function  GetDescription : PCHAR; virtual;
    procedure SetDescription (const Description : PCHAR);
    function  GetMetadata(Domain : PCHAR) : PPCHAR;
    function  SetMetadata(MetaData : PPCHAR; Domain : PCHAR) : PPCHAR;
    function  GetMetadataItem(Name : PCHAR; Domain : PCHAR) : PCHAR;
    function  SetMetadataItem(Name : PCHAR; Value : PCHAR; Domain : PCHAR) : TCPLErr;
    constructor Create;//(obj : TGDALMajorObjectH);
    destructor  Destroy; override;
 end;

//----------------------------------------------------------------------------
  function  GetLastErrorMsg : PCHAR;
  function  GetConfigOption(Key, Default : PCHAR) : PCHAR;
  procedure SetConfigOption(Key, Value : PCHAR);
  procedure AllRegister;

type
  TCPLErrorHandler = procedure (TeErrClass : TCPLErr; err_no : integer; const msg : PCHAR);

const
  ObjIsNULLError = 1001;

//* -------------------------------------------------------------------- */
//*      Helper functions.                                               */
//* -------------------------------------------------------------------- */

  function  GDALGeneralCmdLineProcessor(nArgc : longint; ppapszArgv : PPPCHAR;
                                        nOption : longint
                                       ) : longint; stdcall; external External_Lib name '_GDALGeneralCmdLineProcessor@12';
  procedure GDALSwapWords(pData : POINTER;
                          nWordSize, nWordCount, nWordSkip : longint
                          ); stdcall; external External_Lib name '_GDALSwapWords@16';
  procedure GDALCopyWords(pSrcData : POINTER;
                          eSrcType : TGDALDataType;
                          nSrcPixelOffset : longint;
                          pDstData : POINTER;
                          eDstType : TGDALDataType;
                          nDstPixelOffset : longint;
                          nWordCount : longint
                          ); stdcall; external External_Lib name '_GDALCopyWords@28';
  procedure GDALCopyBits(const pabySrcData : PGByte;
                         nSrcOffset, nSrcStep : longint;
                         pabyDstData : PGByte;
                         nDstOffset, nDstStep : longint;
                         nBitCount, nStepCount : longint
                        ); stdcall; external External_Lib name 'GDALCopyBits';
  function  GDALLoadWorldFile(const pszFilename : PCHAR;
                              padfGeoTransform : PDOUBLE
                              ) : longint; stdcall; external External_Lib name '_GDALLoadWorldFile@8';
  function  GDALReadWorldFile(const pszBaseFilename : PCHAR;
                              const pszExtension : PCHAR;
                              padfGeoTransform : PDOUBLE
                              ) : longint; stdcall; external External_Lib name '_GDALReadWorldFile@12';
  function  GDALWriteWorldFile(const pszBaseFilename : PCHAR;
                               const pszExtension : PCHAR;
                               padfGeoTransform : PDOUBLE
                              ) : longint; stdcall; external External_Lib name '_GDALWriteWorldFile@12';
  function  GDALPackedDMSToDec(dfPacked : double) : double; stdcall; external External_Lib name '_GDALPackedDMSToDec@8';
  function  GDALDecToPackedDMS(dfDec : double) : double; stdcall; external External_Lib name '_GDALDecToPackedDMS@8';
  function  GDALVersionInfo(const pszRequest : PCHAR) : PCHAR; stdcall; external External_Lib name '_GDALVersionInfo@4';
  function  GDALCheckVersion(nVersionMajor, nVersionMinor : longint;
                              const pszCallingComponentName : PCHAR
                             ) : longint; stdcall; external External_Lib name '_GDALCheckVersion@12';

//' ----------------------------------------------------------------------------
//'       CPL
//' ----------------------------------------------------------------------------
  procedure CPLError(eErrClass : TCPLErr; err_no : integer; const fmt : PCHAR); stdcall; external External_Lib name 'CPLError';
  function  CPLGetLastErrorMsg :PCHAR; stdcall; external External_Lib name '_CPLGetLastErrorMsg@0';
  procedure CSLDestroy (const CSLList : PPCHAR);stdcall; external External_Lib name '_CSLDestroy@4';
  function  CPLGetConfigOption (const Key : PCHAR; const Default : PCHAR) : PCHAR; stdcall; external External_Lib name '_CPLGetConfigOption@8';
  procedure CPLSetConfigOption (const Key : PCHAR; const Value : PCHAR); stdcall; external External_Lib name '_CPLSetConfigOption@8';
  procedure CPLAssert(const pszExpression : PCHAR; const pszFile : PCHAR; iLine : integer); stdcall; external External_Lib name '__CPLAssert@12';
  procedure CPLDefaultErrorHandler(eErrClass : TCPLErr; err_no : integer; const msg : PCHAR); stdcall; external External_Lib name '_CPLDefaultErrorHandler@12';
  function  CPLMalloc(const nsize : size_t) : POINTER; stdcall; external External_Lib name 'CPLMalloc';
  procedure CPLErrorReset; stdcall; external External_Lib name '_CPLErrorReset@0';
  procedure CPLGetLastErrorNo; stdcall; external External_Lib name '_CPLGetLastErrorNo@0';
  function  CPLGetLastErrorType : TCPLErr; stdcall; external External_Lib name '_CPLGetLastErrorType@0';
  procedure CPLPopErrorHandler; stdcall; external External_Lib name '_CPLPopErrorHandler@0';
  procedure CPLPushErrorHandler(pfnErrorHandlerNew : TCPLErrorHandler); stdcall; external External_Lib name '_CPLPushErrorHandler@4';
  function  CPLSetErrorHandler(pfnErrorHandlerNew : TCPLErrorHandler) : TCPLErrorHandler; stdcall; external External_Lib name '_CPLSetErrorHandler@4';
  //procedure CPLDebug(const pszCategory : PCHAR; const pszFormat: PCHAR); stdcall; external External_Lib name '';
  //function  CPLFree(pData : POINTER) : longint; stdcall; external External_Lib name 'CPLFree';

// ----------------------------------------------------------------------------
//                           Call Back Function
// ----------------------------------------------------------------------------
  function  GDALDummyProgress(dfComplete : double; const pszMessage : PCHAR;
                              pData : POINTER) : longint; stdcall; external External_Lib name '_GDALDummyProgress@16';
  function  GDALTermProgress(dfComplete : double;const pszMessage : PCHAR;
                             pProgressArg : POINTER) : longint; stdcall; external External_Lib name '_GDALTermProgress@16';
  function  GDALScaledProgress(dfComplete : double; const pszMessage : PCHAR;
        	               pData : POINTER) : longint; stdcall; external External_Lib name '_GDALScaledProgress@16';
  procedure GDALCreateScaledProgress(dfMin : double; dfMax : double;
       	                             pfnProgress : TGDALProgressFunc; pData : POINTER); stdcall; external External_Lib name '_GDALCreateScaledProgress@24';
  procedure GDALDestroyScaledProgress(pData : POINTER); stdcall; external External_Lib name '_GDALDestroyScaledProgress@4';

  // ----------------------------------------------------------------------------
  //                           Pixel Data Type
  // ----------------------------------------------------------------------------

  function  GDALGetDataTypeName (const Datatype : TGDALDataType) : PCHAR ; stdcall; external External_Lib name '_GDALGetDataTypeName@4';
  function  GDALGetDataTypeSize (const Datatype : TGDALDataType) : longint; stdcall; external External_Lib name '_GDALGetDataTypeSize@4';
  function  GDALDataTypeIsComplex (const Datatype : TGDALDataType) : longint; stdcall; external External_Lib name '_GDALDataTypeIsComplex@4';
  function  GDALDataTypeUnion(eType1 : TGDALDatatype; eType2 : TGDALDataType) : TGDALDataType; stdcall; external External_Lib name '_GDALDataTypeUnion@8';

  //*! Translate a GDALColorInterp into a user displayable string. */
  function  GDALGetColorInterpretationName(eInterp : TGDALColorInterp) : PCHAR; stdcall; external External_Lib name 'GDALGetColorInterpretationName';
  //*! Translate a GDALPaletteInterp into a user displayable string. */
  function  GDALGetPaletteInterpretationName(eInterp : TGDALPaletteInterp) : PCHAR; stdcall; external External_Lib name 'GDALGetPaletteInterpretationName';


//* ==================================================================== */
//*      major objects (dataset, and, driver, drivermanager).            */
//* ==================================================================== */
  function  GDALGetMetadataItem (const Handle : TGDALMajorObjectH; const Name : PCHAR;
                                 const Domain : PCHAR) : PCHAR; stdcall; external External_Lib name '_GDALGetMetadataItem@12';
  function  GDALGetMetadata (const Handle : TGDALMajorObjectH; const Domain : PCHAR) : PPCHAR; stdcall; external External_Lib name '_GDALGetMetadata@8';
  function  GDALSetMetadataItem (const Handle : TGDALMajorObjectH; const Name : PCHAR;
                                 const Value : PCHAR; const Domain : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALSetMetadataItem@16';
  function  GDALSetMetadata (const Handle : TGDALMajorObjectH; const MetaData : PPCHAR;
                             const Domain : PCHAR) : PPCHAR; stdcall; external External_Lib name '_GDALSetMetadata@12';
  function  GDALGetDescription (const Handle : TGDALMajorObjectH) : PCHAR; stdcall; external External_Lib name '_GDALGetDescription@4';
  procedure GDALSetDescription (const Handle : TGDALMajorObjectH; const Description : PCHAR); stdcall; external External_Lib name '_GDALSetDescription@8';

//' ----------------------------------------------------------------------------
//'       GDAL Dataset
//' ----------------------------------------------------------------------------
  function  GDALOpen (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH; stdcall; external External_Lib name '_GDALOpen@8';
  function  GDALOpenShared (const Filename : PCHAR; const Access : TGDALAccess) : TGDALDriverH; stdcall; external External_Lib name '_GDALOpenShared@8';
  function  GDALDumpOpenDatasets(var pfp : File) : longint; stdcall; external External_Lib name '_GDALDumpOpenDatasets@4';
  procedure GDALClose (const Handle : TGDALDatasetH); stdcall; external External_Lib name '_GDALClose@4';
  function  GDALGetRasterXSize (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALGetRasterXSize@4';
  function  GDALGetRasterYSize (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALGetRasterYSize@4';
  function  GDALGetRasterCount (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALGetRasterCount@4';
  function  GDALGetRasterBand (const Handle : TGDALDatasetH; const BandNo : longint) : TGDALRasterBandH; stdcall; external External_Lib name '_GDALGetRasterBand@8';
  function  GDALGetProjectionRef (const Handle : TGDALDatasetH) : PCHAR; stdcall; external External_Lib name '_GDALGetProjectionRef@4';
  function  GDALSetProjection (const Handle : TGDALDatasetH; const WKTProj : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALSetProjection@8';
  function  GDALGetGeoTransform (const Handle : TGDALDatasetH; Geotransform : PDouble) : TCPLErr; stdcall; external External_Lib name '_GDALGetGeoTransform@8';
  function  GDALSetGeoTransform (const Handle : TGDALDatasetH; Geotransform : PDouble) : TCPLErr; stdcall; external External_Lib name '_GDALSetGeoTransform@8';
  function  GDALReferenceDataset (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALReferenceDataset@4';
  function  GDALDereferenceDataset (const Handle : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALSetGeoTransform@8';
  procedure GDALFlushCache (const Handle : TGDALDatasetH); stdcall; external External_Lib name '_GDALFlushCache@4';
  function  GDALAddBand(hDataset : TGDALDatasetH; eType : TGDALDataType;
		                   papszOptions : PPCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALAddBand@12';
  function  GDALDatasetAdviseRead(hDS : TGDALDatasetH;
		                  nXOff, nYOff, nXSize,	nYSize, nBufXSize, nBufYSize : longint;
                                  eDT : TGDALDataType; nBandCount : longint;
		                  panBandMap : PINTEGER;
		                  papszOptions : PPCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALDatasetAdviseRead@44';
  procedure GDALGetInternalHandle(hDS : TGDALDatasetH;
		                  const pszRequest : PCHAR); stdcall; external External_Lib name '_GDALGetInternalHandle@8';
  function  GDALBuildOverviews(hDataset : TGDALDatasetH;
		               const pszResampling : PCHAR;
		               nOverviews : longint;
		               panOverviewList : PINTEGER;
		               nListBands : longint;
		               panBandList : PINTEGER;
		               pfnProgress : TGDALProgressFunc;
		               pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALBuildOverviews@32';
  procedure GDALGetOpenDatasets(pahDSList : PPGDALDatasetH;
		                pnCount : PINTEGER); stdcall; external External_Lib name '_GDALGetOpenDatasets@8';
  function  GDALGetAccess(hDS : TGDALDatasetH): longint; stdcall; external External_Lib name  '_GDALGetAccess@4';
  function  GDALCreateDatasetMaskBand(hDS : TGDALDatasetH;
                                      Flags : longint) : TCPLErr; stdcall; external External_Lib name '_GDALCreateDatasetMaskBand@8';
  function  GDALDatasetCopyWholeRaster(hSrcDS : TGDALDatasetH;
		                       hDstDS : TGDALDatasetH;
		                       papszOptions : PPCHAR;
		                       pfnProgress : TGDALProgressFunc;
		                       pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALDatasetCopyWholeRaster@20';
  function  GDALRegenerateOverviews(hSrcBand : TGDALRasterBandH;
		                    nOverviewCount : longint;
		                    pahOvrBands : PGDALRasterBandH;
		                    const pszResampling : PCHAR;
		  	            pfnProgress : TGDALProgressFunc;
		                    pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name 'GDALRegenerateOverviews';
  function  GDALGetDatasetDriver(hDataset : TGDALDatasetH) : TGDALDriverH; stdcall; external External_Lib name '_GDALGetDatasetDriver@4';
  function  GDALGetFileList(hDS : TGDALDatasetH) : PPCHAR; stdcall; external External_Lib name '_GDALGetFileList@4';
  function  GDALDatasetRasterIO(hDS : TGDALDatasetH;
                                eRWFlag : TGDALRWFlag;
                                nXOff, YOff, nXSize: longint;
                                pData : POINTER;
                                nBufXSize, nBufYSize : longint;
                                eBufType : TGDALDataType;
                                nBandCount : longint;
                                panBandMap : PINTEGER;
                                nPixelSpace, nLineSpace, nBandSpace : longint
                               ) : TCPLErr; stdcall; external External_Lib name '_GDALDatasetRasterIO@60';

//============================ GDAL_GCP=======================================*/

function  GDALGetGCPCount(hDS : TGDALDatasetH) : longint; stdcall; external External_Lib name '_GDALGetGCPCount@4';
function  GDALGetGCPProjection(hDS : TGDALDatasetH) : PCHAR; stdcall; external External_Lib name '_GDALGetGCPProjection@4';
function  GDALGetGCPs(hDS : TGDALDatasetH) : PGDAL_GCP; stdcall; external External_Lib name '_GDALGetGCPs@4';
function  GDALSetGCPs(hDS : TGDALDatasetH; nGCPCount : longint;
      	        const pasGCPList : PGDAL_GCP;
      	        const pszGCPProjection : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALSetGCPs@16';

//' ----------------------------------------------------------------------------
//'       GDAL RasterBand
//' ----------------------------------------------------------------------------
  function  GDALGetRasterDataType (const Handle : TGDALRasterBandH) : TGDALDatatype; stdcall; external External_Lib name '_GDALGetRasterDataType@4';
  function  GDALGetRasterBandXSize (const Handle : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetRasterBandXSize@4';
  function  GDALGetRasterBandYSize (const Handle : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetRasterBandYSize@4';
  procedure GDALGetBlockSize (const Handle : TGDALRasterBandH; XSize : PINTEGER; YSize : PINTEGER); stdcall; external External_Lib name '_GDALGetBlockSize@12';
  function  GDALGetRasterNoDataValue (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterNoDataValue@8';
  function  GDALSetRasterNoDataValue (const Handle : TGDALRasterBandH; const NoDataValue : double) : longint; stdcall; external External_Lib name '_GDALSetRasterNoDataValue@12';
  function  GDALGetRasterMinimum (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterMinimum@8';
  function  GDALGetRasterMaximum (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterMaximum@8';
  function  GDALGetRasterOffset (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterOffset@8';
  function  GDALSetRasterOffset (const Handle : TGDALRasterBandH; const Offset : double) : longint; stdcall; external External_Lib name '_GDALSetRasterOffset@12';
  function  GDALGetRasterScale (const Handle : TGDALRasterBandH; pSuccess : PINTEGER) : double; stdcall; external External_Lib name '_GDALGetRasterScale@8';
  function  GDALSetRasterScale (const Handle : TGDALRasterBandH; const NewScale : double) : longint; stdcall; external External_Lib name '_GDALSetRasterScale@12';
  function  GDALRasterIO (const Handle : TGDALRasterBandH; const RWFlag : TGDALRWFlag;
                          const XOff : longint; const YOff : longint;
                          const XSize : longint; const YSize : longint;
                                pData : POINTER; const BufXSize : longint;
                          const BufYSize : longint; const DataType : TGDALDatatype;
                          const PixelSpace : longint; const LineSpace : longint
                          ) : longint; stdcall; external External_Lib name '_GDALRasterIO@48';

  function GDALRasterAdviseRead(hRB : TGDALRasterBandH;
		                nXOff, nYOff, nXSize, nYSize, nBufXSize, nBufYSize,
                                eDT : TGDALDataType;
		                papszOptions : PPCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALRasterAdviseRead@36';
  function GDALReadBlock(hBand : TGDALRasterBandH;
		         nXOff, nYOff : longint;
		         pData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALReadBlock@16';
  function GDALWriteBlock(hBand : TGDALRasterBandH;
                          nXOff, nYOff : longint;
		          pData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALWriteBlock@16';
  function GDALGetRasterAccess(hBand : TGDALRasterBandH) : TGDALAccess; stdcall; external External_Lib name '_GDALGetRasterAccess@4';
  function GDALGetBandNumber(hBand : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetBandNumber@4';
  function GDALGetBandDataset(hBand : TGDALRasterBandH) : TGDALDatasetH; stdcall; external External_Lib name '_GDALGetBandDataset@4';
  function GDALGetRasterColorInterpretation(hBand : TGDALRasterBandH) : TGDALColorInterp; stdcall; external External_Lib name '_GDALGetRasterColorInterpretation@4';
  function GDALSetRasterColorInterpretation(hBand : TGDALRasterBandH;
		                            eColorInterp : TGDALColorInterp) : TCPLErr; stdcall; external External_Lib name '_GDALSetRasterColorInterpretation@8';
  function GDALHasArbitraryOverviews(hBand : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALHasArbitraryOverviews@4';
  function GDALGetOverviewCount(hBand : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetOverviewCount@4';
  function GDALGetOverview(hBand : TGDALRasterBandH; i : longint) : TGDALRasterBandH; stdcall; external External_Lib name '_GDALGetOverview@8';
  function GDALGetRasterCategoryNames(hBand : TGDALRasterBandH) : PPCHAR; stdcall; external External_Lib name '_GDALGetRasterCategoryNames@4';
  function GDALSetRasterCategoryNames(hBand : TGDALRasterBandH;
  		                      papszNames : PPCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALSetRasterCategoryNames@8';
  function GDALGetRasterStatistics(hBand : TGDALRasterBandH;
  		                   bApproxOK, bForce : longint;
  		                   pdfMin, pdfMax, pdfMean, pdfStdDev : PDOUBLE
  	                          ) : TCPLErr; stdcall; external External_Lib name '_GDALGetRasterStatistics@28';
  function  GDALComputeRasterStatistics(hBand : TGDALRasterBandH;
                                        bApproxOK : longint;
                                        pdfMin, pdfMax, pdfMean, pdfStdDev : PDOUBLE;
                                        pfnProgress : TGDALProgressFunc;
                                        pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALComputeRasterStatistics@32';
  function  GDALSetRasterStatistics(hBand : TGDALRasterBandH;
                                    bApproxOK, bForce : longint;
                                    pdfMin, pdfMax, pdfMean, pdfStdDev : double
  	                            ) : TCPLErr; stdcall; external External_Lib name '_GDALSetRasterStatistics@28';
  function  GDALGetRasterUnitType(hBand : TGDALRasterBandH) : PCHAR; stdcall; external External_Lib name '_GDALGetRasterUnitType@4';
  procedure GDALComputeRasterMinMax(hBand : TGDALRasterBandH;
                                    bApproxOK : longint;
                                    adfMinMax : TDoubleArray2); stdcall; external External_Lib name '_GDALComputeRasterMinMax@12';
  function  GDALFlushRasterCache(hBand : TGDALRasterBandH) : TCPLErr; stdcall; external External_Lib name '_GDALFlushRasterCache@4';
  function  GDALGetRasterHistogram(hBand : TGDALRasterBandH;
                                   dfMin, dfMax : double;
                                   nBuckets : longint;
                                   panHistogram : PINTEGER;
                                   bIncludeOutOfRange, bApproxOK : longint;
                                   pfnProgress : TGDALProgressFunc;
                                   pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALGetRasterHistogram@44';
  function  GDALGetDefaultHistogram(hBand : TGDALRasterBandH;
                                    pdfMin, pdfMax : PDOUBLE;
                                    pnBuckets : PINTEGER;
                                    ppanHistogram : PPINTEGER;
                                    bForce : longint;
                                    pfnProgress : TGDALProgressFunc;
                                    pProgressData : POINTER) : TCPLErr; stdcall; external External_Lib name '_GDALGetDefaultHistogram@32';
  function  GDALSetDefaultHistogram(hBand : TGDALRasterBandH;
                                    dfMin, dfMax : double;
                                    nBuckets : longint;
                                    panHistogram : PINTEGER) : TCPLErr; stdcall; external External_Lib name '_GDALSetDefaultHistogram@28';
  function  GDALGetRasterSampleOverview(hBand : TGDALRasterBandH;
                                        nDesiredSamples : longint
                                       ) : TGDALRasterBandH; stdcall; external External_Lib name '_GDALGetRasterSampleOverview@8';
  function  GDALFillRaster(hBand : TGDALRasterBandH;
                           dfRealValue, dfImaginaryValue : double
                          ) : TCPLErr; stdcall; external External_Lib name '_GDALFillRaster@20';
  function  GDALGetDefaultRAT(hBand : TGDALRasterBandH) : TGDALRasterAttributeTableH; external External_Lib name '_GDALGetDefaultRAT@4';
  function  GDALSetDefaultRAT(hBand : TGDALRasterBandH;
                              hRAt : TGDALRasterAttributeTableH
                             ) : TCPLErr; stdcall; external External_Lib name '_GDALSetDefaultRAT@8';
  function  GDALAddDerivedBandPixelFunc(const pszFuncName : PCHAR;
                                        pfnNewFunction : TGDALDerivedPixelFunc
                                       ) : TCPLErr; stdcall; external External_Lib name '_GDALAddDerivedBandPixelFunc@8';
  function  GDALGetMaskBand(hBand : TGDALRasterBandH) : TGDALRasterBandH; stdcall; external External_Lib name '_GDALGetMaskBand@4';
  function  GDALGetMaskFlags(hBand : TGDALRasterBandH) : longint; stdcall; external External_Lib name '_GDALGetMaskFlags@4';
  function  GDALCreateMaskBand(hBand : TGDALRasterBandH; nFlags : longint
                              ) : TCPLErr; stdcall; external External_Lib name '_GDALCreateMaskBand@8';



//' ----------------------------------------------------------------------------
//'       GDALDriver
//' ----------------------------------------------------------------------------
  procedure GDALAllRegister ; stdcall; external External_Lib name '_GDALAllRegister@0';
  function  GDALGetDriverByName (const Filename : PCHAR) : TGDALDriverH; stdcall; external External_Lib name '_GDALGetDriverByName@4';
  function  GDALGetDriverCount : longint; stdcall; external External_Lib name '_GDALGetDriverCount@0';
  function  GDALGetDriver (const DriverIndex : longint) : TGDALDriverH; stdcall; external External_Lib name '_GDALGetDriver@4';
  procedure GDALDestroyDriverManager ; stdcall; external External_Lib name '_GDALDestroyDriverManager@0';
  function  GDALCreate (const Handle : TGDALDriverH; const Filename : PCHAR;
                        const XSize : longint; const YSize : longint; const BandCount : longint;
                        const DataType : TGDALDataType; const Options : PPCHAR
                        ) : TGDALDatasetH; stdcall; external External_Lib name '_GDALCreate@28';
  function  GDALCreateCopy (const Handle : TGDALDriverH; const Filename : PCHAR;
                            const SrcDS : TGDALDatasetH; const ApproxOK : longint; const Options : PPCHAR;
                            const ProgressFunc : TGDALDatasetH; const ProgressArg : longint) : TGDALDatasetH; stdcall; external External_Lib name '_GDALCreateCopy@28';
  procedure GDALDestroyDriver(hDriver : TGDALDriverH); stdcall; external External_Lib name '_GDALDestroyDriver@4';
  function  GDALRegisterDriver(hDriver : TGDALDriverH) : longint; stdcall; external External_Lib name '_GDALRegisterDriver@4';
  procedure GDALDeregisterDriver(hDriver : TGDALDriverH); stdcall; external External_Lib name '_GDALDeregisterDriver@4';
  function  GDALValidateCreationOptions(hDriver : TGDALDriverH;
		                        papszCreationOptions : PPCHAR) : longint; stdcall; external External_Lib name '_GDALValidateCreationOptions@8';
  function  GDALGetDriverShortName(hDriver : TGDALDriverH) : PCHAR; stdcall; external External_Lib name '_GDALGetDriverShortName@4';
  function  GDALGetDriverLongName(hDriver : TGDALDriverH) : PCHAR; stdcall; external External_Lib name '_GDALGetDriverLongName@4';
  function  GDALGetDriverHelpTopic(hDriver : TGDALDriverH) : PCHAR; stdcall; external External_Lib name '_GDALGetDriverHelpTopic@4';
  function  GDALGetDriverCreationOptionList(hDriver : TGDALDriverH) : PCHAR; stdcall; external External_Lib name '_GDALGetDriverCreationOptionList@4';
  function  GDALGCPsToGeoTransform(nGCPCount : longint; const pasGCPs : PGDAL_GCP;
		                   padfGeoTransform : PDOUBLE;
                                   bApproxOK : longint) : longint; stdcall; external External_Lib name '_GDALGCPsToGeoTransform@16';
  function  GDALInvGeoTransform(gt_in : PDOUBLE; gt_out : PDOUBLE) : longint; stdcall; external External_Lib name '_GDALInvGeoTransform@8';
  procedure GDALApplyGeoTransform(padfGeoTransform : PDOUBLE;
		                  dfPixel : double;
		                  dfLine : double;
		                  pdfGeoX : PDOUBLE;
		                  pdfGeoY : PDOUBLE); stdcall; external External_Lib name '_GDALApplyGeoTransform@28';

  function  GDALDeleteDataset(hDriver : TGDALDriverH; const pszFilename : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALDeleteDataset@8';
  function  GDALRenameDataset(hDriver : TGDALDriverH; const pszNewName : PCHAR;
                              const pszOldName : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALRenameDataset@12';
  function  GDALCopyDatasetFiles(hDriver : TGDALDriverH; const pszNewName : PCHAR;
                                 const pszOldName : PCHAR) : TCPLErr; stdcall; external External_Lib name '_GDALCopyDatasetFiles@12';
  function  GDALIdentifyDriver(const pszFilename : PCHAR;
		               papszFileList : PPCHAR) : TGDALDriverH; stdcall; external External_Lib name '_GDALIdentifyDriver@8';



//' ----------------------------------------------------------------------------
//'       GDALColorTable
//' ----------------------------------------------------------------------------
  function  GDALCreateColorTable (const PaletteInterp : TGDALPaletteInterp) : TGDALColorTableH; stdcall; external External_Lib name '_GDALCreateColorTable@4';
  procedure GDALDestroyColorTable (const Handle : TGDALColorTableH); stdcall; external External_Lib name '_GDALDestroyColorTable@4';
  function  GDALCloneColorTable (const Handle : TGDALColorTableH) : TGDALColorTableH; stdcall; external External_Lib name '_GDALCloneColorTable@4';
  function  GDALGetPaletteInterpretation (const Handle : TGDALColorTableH) : longint; stdcall; external External_Lib name '_GDALGetPaletteInterpretation@4';
  function  GDALGetColorEntryCount (const Handle : TGDALColorTableH) : longint; stdcall; external External_Lib name '_GDALGetColorEntryCount@4';
  function  GDALGetColorEntryAsRGB (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                          ColorEntry : Integer) : longint; stdcall; external External_Lib name '_GDALGetColorEntryAsRGB@12';
  function  GDALGetColorEntry (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                     ColorEntry : Integer) : longint; stdcall; external External_Lib name '_GDALGetColorEntry@12';
  procedure GDALSetColorEntry (const Handle : TGDALColorTableH; const ColorIndex : longint;
                                     ColorEntry : Integer); stdcall; external External_Lib name '_GDALSetColorEntry@12';
  procedure GDALCreateColorRamp(hTable : TGDALColorTableH;
                                nStartIndex : longint;
                                const psStartColor : PGDALColorEntry;
                                nEndIndex : longint;
                                const psEndColor : PGDALColorEntry
                               ); stdcall; external External_Lib name '_GDALCreateColorRamp@20';
  function GDALGetRasterColorTable (const Handle : TGDALRasterBandH) : TGDALColorTableH; stdcall; external External_Lib name '_GDALGetRasterColorTable@4';
  function GDALSetRasterColorTable (const BandHandle : TGDALRasterBandH; const TableHandle : TGDALColorTableH) : TCPLErr; stdcall; external External_Lib name '_GDALSetRasterColorTable@8';


//* ==================================================================== */
//*      Raster Attribute Table                                          */
//* ==================================================================== */
  function  GDALCreateRasterAttributeTable : TGDALRasterAttributeTableH; stdcall; external External_Lib name '_GDALCreateRasterAttributeTable@0';
  procedure GDALDestroyRasterAttributeTable(hRAT : TGDALRasterAttributeTableH ); stdcall; external External_Lib name '_GDALDestroyRasterAttributeTable@4';
  function  GDALRATGetColumnCount(hRAT : TGDALRasterAttributeTableH ) : longint; stdcall; external External_Lib name '_GDALRATGetColumnCount@4';
  function  GDALRATGetNameOfCol(hRAT : TGDALRasterAttributeTableH;
                                iCol : longint ) : PCHAR; stdcall; external External_Lib name '_GDALRATGetNameOfCol@8';
  function  GDALRATGetUsageOfCol(hRAT : TGDALRasterAttributeTableH;
                                iCol : longint) : TGDALRATFieldUsage; stdcall; external External_Lib name '_GDALRATGetUsageOfCol@8';
  function  GDALRATGetTypeOfCol(hRAT : TGDALRasterAttributeTableH;
                                iCol : longint) : TGDALRATFieldType; stdcall; external External_Lib name '_GDALRATGetTypeOfCol@8';
  function  GDALRATGetColOfUsage(hRAT : TGDALRasterAttributeTableH;
                                 eUsage : TGDALRATFieldUsage ) : longint; stdcall; external External_Lib name '_GDALRATGetColOfUsage@8';
  function  GDALRATGetRowCount( hRAT : TGDALRasterAttributeTableH) : longint; stdcall; external External_Lib name '_GDALRATGetRowCount@4';
  function  GDALRATGetValueAsString(hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField : longint) : PCHAR; stdcall; external External_Lib name '_GDALRATGetValueAsString@12';
  function  GDALRATGetValueAsInt(hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField, nValue : longint) : longint; stdcall; external External_Lib name '_GDALRATGetValueAsInt@12';
  function  GDALRATGetValueAsDouble(hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField : longint) : double; stdcall; external External_Lib name '_GDALRATGetValueAsDouble@12';
  procedure GDALRATSetValueAsString(hRAT : TGDALRasterAttributeTableH;
                                    iRow, iField : longint;
                                    const pszValue : PCHAR); stdcall; external External_Lib name '_GDALRATSetValueAsString@16';
  procedure GDALRATSetValueAsInt(hRAT : TGDALRasterAttributeTableH;
                                 iRow, iField, nValue : longint); stdcall; external External_Lib name '_GDALRATSetValueAsInt@16';
  procedure GDALRATSetValueAsDouble(hRAT : TGDALRasterAttributeTableH;
                                    iRow, iField : longint;
                                    dfValue : double ); stdcall; external External_Lib name '_GDALRATSetValueAsDouble@20';
  procedure GDALRATSetRowCount(hRAT : TGDALRasterAttributeTableH;
                               nNewCount : longint); stdcall; external External_Lib name '_GDALRATSetRowCount@8';
  function  GDALRATCreateColumn(hRAT : TGDALRasterAttributeTableH;
            pszFieldName : PCHAR;
            eFieldType : TGDALRATFieldType;
            eFieldUsage : TGDALRATFieldUsage) : TCPLErr; stdcall; external External_Lib name '_GDALRATCreateColumn@16';
  function  GDALRATSetLinearBinning(hRAT : TGDALRasterAttributeTableH;
                                    dfRow0Min, dfBinSize : double ) : TCPLErr; stdcall; external External_Lib name '_GDALRATSetLinearBinning@20';
  function  GDALRATGetLinearBinning(hRAT : TGDALRasterAttributeTableH;
                                    pdfRow0Min, pdfBinSize :PDOUBLE) : longint; stdcall; external External_Lib name '_GDALRATGetLinearBinning@12';
  function  GDALRATInitializeFromColorTable(hRAT : TGDALRasterAttributeTableH;
                                            hCT : TGDALColorTableH ) : TCPLErr; stdcall; external External_Lib name '_GDALRATInitializeFromColorTable@8';
  function  GDALRATTranslateToColorTable(hRAT : TGDALRasterAttributeTableH;
                                         nEntryCount : longint) : TGDALColorTableH; stdcall; external External_Lib name '_GDALRATInitializeFromColorTable@8';
  procedure GDALRATDumpReadable(hRAT : TGDALRasterAttributeTableH; var fp : FILE ); stdcall; external External_Lib name '_GDALRATDumpReadable@8';
  function  GDALRATClone(hRAT : TGDALRasterAttributeTableH ) : TGDALRasterAttributeTableH; stdcall; external External_Lib name '_GDALRATClone@4';
  function  GDALRATGetRowOfValue(hRAT : TGDALRasterAttributeTableH;
                                 dfValue : double ) : longint; stdcall; external External_Lib name '_GDALRATGetRowOfValue@12';


implementation
{
function SrcVal(papoSource : PPOINTER;
		eSrcType : TGDALDatatype;
		ii : longint) : variant;

var
  val :variant;
begin
  case eSrcType of
    GDT_Byte    : val.vbyte := GByte(papoSource[ii]);
    GDT_Float32 : val.vsingle := GFloat32(papoSource[ii]);
    GDT_Float64 : val.vdouble := GFloat64(papoSource[ii]);
    GDT_Int32   : val.vinteger := GInt32(papoSource[ii]);
    GDT_UInt16  : val.vword := GUInt16(papoSource[ii]);
    GDT_Int16   : val.vsmallint := GInt16(papoSource[ii]);
    GDT_UInt32  : val.vlongword := GUInt32(papoSource[ii]);
    GDT_CInt16  : val.vword := GInt16(papoSource[ii * 2]);
    GDT_CInt32  : val.vlongword := GCint32(papoSource[ii * 2]);
    GDT_CFloat32: val.vsingle := GCFloat32(papoSource[ii * 2]);
    GDT_CFloat64: val.vdouble := GCFloat64(papoSource[ii * 2]);
  else
    val := 0;
  end;
end;
}

//----------------------------------------------------------------------------//
//                       TGDALMajorObject                                     //
//----------------------------------------------------------------------------//

constructor TGDALMajorObject.Create;
begin
  inherited create;
  FObj := NIL;
end;

destructor TGDALMajorObject.Destroy;
begin
  inherited Destroy;
end;

function  TGDALMajorObject.GetDescription : PCHAR;
begin
  result := GDALCore.GDALGetDescription(FObj);
end;

procedure TGDALMajorObject.SetDescription (const Description : PCHAR);
begin
  GDALCore.GDALSetDescription(FObj, Description);
end;

//' ----------------------------------------------------------------------------
function TGDALMajorObject.GetMetadata(Domain : PCHAR) : PPCHAR;
begin
  if (FObj <> NIL) then
    result := GDALCore.GDALGetMetadata(FObj, Domain);
end;

//' ----------------------------------------------------------------------------
function TGDALMajorObject.SetMetadata(MetaData : PPCHAR; Domain : PCHAR) : PPCHAR;
begin
  if (FObj <> NIL) then
    result := GDALCore.GDALSetMetadata(FObj, MetaData, Domain)
  else
    result := PPCHAR(GDALCore.ObjIsNULLError);
end;

//' ----------------------------------------------------------------------------
function TGDALMajorObject.GetMetadataItem(Name : PCHAR; Domain : PCHAR) : PCHAR;
begin
  if (FObj <> NIL) then
    result := GDALCore.GDALGetMetadataItem(FObj, Name, Domain)
  else
    result := NIL;

end;

//' -------------------------ECW SDK---------------------------------------------------
function TGDALMajorObject.SetMetadataItem(Name : PCHAR; Value : PCHAR; Domain : PCHAR) : TCPLErr;
begin
  if (FObj <> NIL) then
    result := GDALCore.GDALSetMetadataItem(FObj, Name, Value, Domain)
  else
    result := CE_None;
end;


//----------------------------------------------------------------------------
function GetLastErrorMsg : PCHAR;
begin
   result := GDALCore.CPLGetLastErrorMsg;
end;

//' ----------------------------------------------------------------------------
function GetConfigOption(Key : PCHAR; Default : PCHAR) : PCHAR;
begin
  result := GDALCore.CPLGetConfigOption(Key, Default);
end;

//' ----------------------------------------------------------------------------
procedure SetConfigOption(Key : PCHAR; Value : PCHAR);
begin
  GDALCore.CPLSetConfigOption(Key, Value);
end;

//' ----------------------------------------------------------------------------
procedure AllRegister;
begin
  GDALCore.GDALAllRegister;
end;


end.

